import {
  DecoratorType,
  Type,
} from '@overflow/commons/core/type';

import * as TypeUtil from '@overflow/commons/core/type/util';

import * as Constants from './constants';
import {
  Annotation,
} from './annotation';

export class Decorator {
  public static create = (AnnotationClass: Type<Annotation>) => {

    return (...handlerArgs: any[]) => {
      let annotation: Annotation = new AnnotationClass(...handlerArgs);
      let name: string = AnnotationClass.name;

      return (...decoratorArgs: any[]) => {
        let decoratorType: DecoratorType = TypeUtil.getDecoratorType(decoratorArgs);

        // let type = typeof decoratorArgs[0] === 'function' ? decoratorArgs[0].prototype : decoratorArgs[0];
        // let clazz = TypeUtil.getClass(decoratorArgs[0]);

        switch(decoratorType) {
        case DecoratorType.CLASS:
          if (typeof(annotation.classDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Class.`);
          }

          return annotation.classDecorator.call(annotation, decoratorArgs[0]);
        case DecoratorType.PROPERTY:
          if (typeof(annotation.propertyDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Property.`);
          }

          return annotation.propertyDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1]);
        case DecoratorType.METHOD:
          if (typeof(annotation.methodDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Method.`);
          }

          return annotation.methodDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        case DecoratorType.PARAMETER:
          if (typeof(annotation.parameterDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Parameter.`);
          }

          return annotation.parameterDecorator.call(annotation, decoratorArgs[0], decoratorArgs[1], decoratorArgs[2]);
        default:
        }
      };
    };
  }

}
